# -*- coding: utf-8 -*-

import os
import platform
import sys
import tempfile
import tarfile

base = os.getcwd()

PYTHON_VERSION = ".".join((str(sys.version_info[0]), str(sys.version_info[1])))
OS_VERSION = tuple(int(v) for v in platform.mac_ver()[0].split('.'))

# distribution version
DISTRIBUTION_VERSION = 1

# dependencies name and version
bzr = {
    'name':'bzr',
    'version':'2.4.2',
    'docversion':'2.3',
    'destdir':'bzr',
    'url':'http://launchpad.net/bzr/%(version).3s/%(version)s/+download/bzr-%(version)s.tar.gz',
}

# packages
paramiko = {
    'name':'paramiko',
    'version':'1.7.6',
    'destdir':'paramiko',
    'url':'https://github.com/robey/paramiko/tarball/release-%(version)s',
    'patches':[('rng.py.patch-py%s' % PYTHON_VERSION, 5),
               ('transport.py.patch', 0)],
}
pycrypto = {
    'name':'pycrypto',
    'version':'2.3',
    'destdir':'pycrypto',
    'url':'http://www.pycrypto.org/files/pycrypto-%(version)s.tar.gz',
}
pyqt = {
    'name':'PyQt-mac-gpl',
    'version':'4.8.5',
    'destdir':'pyqt',
    'url':'http://www.riverbankcomputing.co.uk/static/Downloads/PyQt4/PyQt-mac-gpl-%(version)s.tar.gz',
}
sip = {
    'name':'sip',
    'version':'4.12.4',
    'destdir':'sip',
    'url':'http://www.riverbankcomputing.co.uk/static/Downloads/sip4/sip-%(version)s.tar.gz',
}
launchpadlib = {
    'name':'launchpadlib',
    'version':'1.9.4',
    'destdir':'launchpadlib',
    'url':'http://launchpad.net/launchpadlib/trunk/%(version)s/+download/launchpadlib-%(version)s.tar.gz',
}
python_fastimport = {
    'name':'python-fastimport',
    'version':'0.9.0',
    'destdir':'python-fastimport',
    'url':'http://launchpad.net/python-fastimport/trunk/%(version)s/+download/python-fastimport-%(version)s.tar.gz',
}

# plugins
bzrtools = {
    'name':'bzrtools',
    'revno':'tag:release-2.4.0',
    'destdir':'bzrtools',
    'branch':'lp:bzrtools',
    #'url':'http://launchpad.net/bzrtools/stable/%(version)s/+download/bzrtools-%(version)s.tar.gz',
}
colo = {
    'name':'bzr-colo',
    'version':'0.3.0',
    'destdir':'colo',
    'url':'http://launchpad.net/bzr-colo/trunk/%(version)s/+download/bzr-colo-%(version)s.tar.gz',
}
email = {
    'name':'bzr-email',
    'revno':'49',
    'destdir':'email',
    'branch':'lp:bzr-email',
}
explorer = {
    'name':'bzr-explorer',
    'destdir':'explorer',
    'revno':'tag:release-1.2.1',
    'branch':'lp:bzr-explorer',
    #'url':'http://launchpad.net/bzr-explorer/%(version).3s/%(version)s/+download/bzr-explorer-%(version)s.tar.gz',
}
extmerge = {
    'name':'bzr-extmerge',
    'revno':'16',
    'destdir':'extmerge',
    'branch':'lp:bzr-extmerge',
}
fastimport = {
    'name':'bzr-fastimport',
    'revno':'327',
    'destdir':'fastimport',
    'branch':'lp:bzr-fastimport',
    #'url':'http://launchpad.net/bzr-fastimport/trunk/%(version)s/+download/bzr-fastimport-%(version)s.tar.gz',
}
keychain = {
    'name':'bzr-keychain',
    'revno':'15',
    'destdir':'keychain',
    'branch':'lp:bzr-keychain',
}
loom = {
    'name':'bzr-loom',
    'revno':'140',
    'destdir':'loom',
    'branch':'lp:bzr-loom',
}
pipeline = {
    'name':'bzr-pipeline',
    'revno':'203',
    'destdir':'pipeline',
    'branch':'lp:bzr-pipeline',
}
qbzr = {
    'name':'qbzr',
    'revno':'1425',
    'destdir':'qbzr',
    'branch':'lp:qbzr/0.21',
    #'url':'http://launchpad.net/qbzr/%(version).4s/%(version)s/+download/qbzr-%(version)s.tar.gz'
}
rewrite = {
    'name':'bzr-rewrite',
    'revno':'tag:bzr-rewrite-0.6.2',
    'destdir':'rewrite',
    'branch':'lp:bzr-rewrite',
    #'url':'http://launchpad.net/bzr-rewrite/trunk/%(version)s/+download/bzr-rewrite-%(version)s.tar.gz',
}
subvertpy = {
    'name':'subvertpy',
    'destdir':'subvertpy',
    'revno':'tag:subvertpy-0.8.3',
    'branch':'lp:subvertpy',
}
svn = {
    'name':'bzr-svn',
    'revno':'tag:bzr-svn-1.1.0',
    'destdir':'svn',
    'branch':'lp:bzr-svn',
}
upload = {
    'name':'bzr-upload',
    'destdir':'upload',
    'revno':'86',
    'branch':'lp:bzr-upload',
}
xmloutput = {
    'name':'bzr-xmloutput',
    'revno':'157',
    'destdir':'xmloutput',
    'branch':'lp:bzr-xmloutput',
}

# list of packages and plugins to build
# build in order as they appear here
packages = [
    paramiko,
    pycrypto,
    bzr,
    sip,
    pyqt,
    launchpadlib,
    python_fastimport,
]
plugins = [
    email,
    rewrite,
    bzrtools,
    upload,
    xmloutput,
    extmerge,
    qbzr,
    explorer,
    colo,
    fastimport,
    loom,
    pipeline,
]

docs = {
    'doc/developers/_build/latex': {
        'bzr-en-architecture-overview.pdf'      : 'Developers/Architecture Overview.pdf',
        'bzr-en-developer-guide.pdf'            : 'Developers/Developer Guide.pdf',
        'bzr-en-integration-guide.pdf'          : 'Developers/Integration Guide.pdf',
        'bzr-en-testing-guide.pdf'              : 'Developers/Testing Guide.pdf',
    },
    'doc/en/_build/latex': {
        'bzr-en-admin-guide.pdf'                : "English/Administrator's Guide.pdf",
        'bzr-en-tutorial-centralized.pdf'       : 'English/Centralized Workflow Tutorial.pdf',
        'bzr-en-tutorial-mini.pdf'              : 'English/Bazaar in Five Minutes.pdf',
        'bzr-en-tutorial-with-launchpad.pdf'    : 'English/Using Bazaar with Launchpad.pdf',
        'bzr-en-tutorial.pdf'                   : 'English/Tutorial.pdf',
        'bzr-en-upgrade-guide.pdf'              : 'English/Upgrade Guide.pdf',
        'bzr-en-user-guide.pdf'                 : 'English/User Guide.pdf',
        'bzr-en-user-reference.pdf'             : 'English/User Reference.pdf',
        'bzr-en-whats-new.pdf'                  : "English/What's New.pdf",
    },
    'doc/en/_static/en': {
        'bzr-en-quick-reference.pdf'            : 'English/Quick Reference.pdf',
    },
    'doc/es/_build/latex': {
        'bzr-es-tutorial-mini.pdf'              : 'Español/Bazaar en Cinco Minutos.pdf',
        'bzr-es-user-guide.pdf'                 : 'Español/Guia de Usuario.pdf',
    },
    'doc/es/_static/es': {
        'bzr-es-quick-reference.pdf'            : 'Español/Referencia rápida.pdf',
    },
    'doc/ru/_static/ru': {
        'bzr-ru-quick-reference.pdf'            : 'Русский/Краткий справочник.pdf',
    },
    'doc/ja/_build/html': {
        '.'                                     : '日本語',
    }
}

## end of configurable stuff ##

if OS_VERSION[1] >= 5:
    plugins.extend([svn, keychain])
    packages.append(subvertpy)

# utilities
def extract(src, dest):
    tmpdir = tempfile.mkdtemp()
    tar = tarfile.open(src)
    tar.extractall(tmpdir)
    tar.close()

    tmp = None
    for entry in os.listdir(tmpdir):
        if os.path.isdir(os.path.join(tmpdir,entry)):
            tmp = entry
            break

    if tmp is None:
        raise Exception('tarfile %s is empty' % src)
    tmp_dest = os.path.join(tmpdir, dest)
    os.rename(os.path.join(tmpdir, tmp), tmp_dest)
    return tmp_dest

def get_plugin_fullname(plugin):
    name = plugin['name']
    version = None
    if plugin.get('version', None) is None:
        if plugin['revno'] == 'tip':
            version = 'tip'
        elif plugin['revno'][:4] == 'tag:':
            version = plugin['revno'][4:]
        else:
            version = 'r' + plugin['revno']
    else:
        version = plugin['version']
    return '%s-%s' % (name, version)

def get_tarfile(plugin):
    return get_plugin_fullname(plugin) + '.tar.gz'
